@@ -3,7 +3,7 @@
 //
 // MIT License
 //
-// Copyright (c) 2014-2018 Michael Truog <mjtruog at protonmail dot com>
+// Copyright (c) 2014-2019 Michael Truog <mjtruog at protonmail dot com>
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -73,15 +73,40 @@ var packUint32big = function packUint32big (value) {
                        value & 0xff]);
 };
 var nodejs_version = process.versions['node'].split('.').map(s => parseInt(s));
-var nodejsVersionAfter = function nodejsVersionAfter (s) {
+var nodejsVersionAfter = function nodejsVersionAfter (s, include) {
     var v = s.split('.').map(s => parseInt(s));
     for (var i = 0; i < v.length; i++) {
         if (nodejs_version[i] > v[i]) {
             return true;
         }
+        if (nodejs_version[i] < v[i]) {
+            return false;
+        }
     }
-    return false;
+    return include;
 };
+if (nodejsVersionAfter('10.0.0',true)) {
+    var originalEmitWarning = process.emitWarning;
+    process.emitWarning = function(warning, type, code, ctor) {
+        if (code === 'DEP0097') {
+            // Ignore the stderr output of the runtime deprecation:
+            // "Using a domain property in MakeCallback is deprecated.
+            //  Use the async_context variant of MakeCallback or the
+            //  AsyncResource class instead."
+            //
+            // Until the exception handling of the domain module is
+            // provided by a different module (like async_hooks) or
+            // exceptions (throw/try/catch) are removed from Javascript,
+            // the use of the domain module needs to remain
+            // (to remain consistent with other CloudI API implementations).
+            // The stderr runtime deprecation information is ignored
+            // because it provides no useful information and is only an
+            // annoyance as a false negative.
+            return;
+        }
+        originalEmitWarning(warning, type, code, ctor);
+    };
+}
 
 var InvalidInputException = function InvalidInputException () {
     var error = new Error('Invalid Input');
@@ -213,7 +238,7 @@ CloudI.API = function API (thread_index, callback) {
     else {
         throw new InvalidInputException();
     }
-    if (nodejsVersionAfter('0.12.1')) {
+    if (nodejsVersionAfter('0.12.1',false)) {
         API._s_in = new net.Socket({fd: (thread_index + 3),
                                     readable: true,
                                     writable: true});
@@ -1053,6 +1078,17 @@ CloudI.API.prototype.poll = function (callback, timeout) {
     }, 0);
 };
 
+CloudI.API.prototype.shutdown = function (callback, reason) {
+    callback = typeof callback !== 'undefined' ?
+               callback : function () {};
+    reason = typeof reason !== 'undefined' ?
+             reason : "";
+    this._poll_wait(function (API) {
+        API._send([new Erlang.OtpErlangAtom('shutdown'), reason]);
+        callback();
+    });
+};
+
 CloudI.API.prototype._text_key_value_parse = function (text) {
     var result = {};
     var data = text.toString('binary').split('\x00');
