@@ -3,7 +3,7 @@
 //
 // MIT License
 //
-// Copyright (c) 2014-2017 Michael Truog <mjtruog at protonmail dot com>
+// Copyright (c) 2014-2019 Michael Truog <mjtruog at protonmail dot com>
 // Copyright (c) 2009-2013 Dmitry Vasiliev <dima@hlabs.org>
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
@@ -494,6 +494,109 @@ var hex = function hex(buffer) {
             assert.equal(term, -6618611909121);
         });
     }).call(this);
+    (function test_binary_to_term_pid () {
+        var pid_old_binary = (
+            '\x83\x67\x64\x00\x0D\x6E\x6F\x6E\x6F\x64\x65\x40\x6E\x6F' +
+            '\x68\x6F\x73\x74\x00\x00\x00\x4E\x00\x00\x00\x00\x00'
+        );
+        Erlang.binary_to_term(pid_old_binary,
+                              function(err, term) {
+            assert.strictEqual(err, undefined);
+            assert.ok(term instanceof Erlang.OtpErlangPid);
+            Erlang.term_to_binary(term,
+                                  function(err, binary) {
+                assert.strictEqual(err, undefined);
+                assert.equal(binary.toString('binary'),
+                    '\x83gs\rnonode@nohost\x00\x00\x00N' +
+                    '\x00\x00\x00\x00\x00');
+            });
+        });
+        var pid_new_binary = (
+            '\x83\x58\x64\x00\x0D\x6E\x6F\x6E\x6F\x64\x65\x40\x6E\x6F\x68' +
+            '\x6F\x73\x74\x00\x00\x00\x4E\x00\x00\x00\x00\x00\x00\x00\x00'
+        );
+        Erlang.binary_to_term(pid_new_binary,
+                              function(err, term) {
+            assert.strictEqual(err, undefined);
+            assert.ok(term instanceof Erlang.OtpErlangPid);
+            Erlang.term_to_binary(term,
+                                  function(err, binary) {
+                assert.strictEqual(err, undefined);
+                assert.equal(binary.toString('binary'),
+                    '\x83Xs\rnonode@nohost\x00\x00\x00N' +
+                    '\x00\x00\x00\x00\x00\x00\x00\x00');
+            });
+        });
+    }).call(this);
+    (function test_binary_to_term_port () {
+        var port_old_binary = (
+            '\x83\x66\x64\x00\x0D\x6E\x6F\x6E\x6F\x64\x65\x40\x6E\x6F\x68' +
+            '\x6F\x73\x74\x00\x00\x00\x06\x00'
+        );
+        Erlang.binary_to_term(port_old_binary,
+                              function(err, term) {
+            assert.strictEqual(err, undefined);
+            assert.ok(term instanceof Erlang.OtpErlangPort);
+            Erlang.term_to_binary(term,
+                                  function(err, binary) {
+                assert.strictEqual(err, undefined);
+                assert.equal(binary.toString('binary'),
+                    '\x83fs\rnonode@nohost\x00\x00\x00\x06\x00');
+            });
+        });
+        var port_new_binary = (
+            '\x83\x59\x64\x00\x0D\x6E\x6F\x6E\x6F\x64\x65\x40\x6E\x6F\x68' +
+            '\x6F\x73\x74\x00\x00\x00\x06\x00\x00\x00\x00'
+        );
+        Erlang.binary_to_term(port_new_binary,
+                              function(err, term) {
+            assert.strictEqual(err, undefined);
+            assert.ok(term instanceof Erlang.OtpErlangPort);
+            Erlang.term_to_binary(term,
+                                  function(err, binary) {
+                assert.strictEqual(err, undefined);
+                assert.equal(binary.toString('binary'),
+                    '\x83Ys\rnonode@nohost\x00\x00\x00\x06' +
+                    '\x00\x00\x00\x00');
+            });
+        });
+    }).call(this);
+    (function test_binary_to_term_ref () {
+        var ref_new_binary = (
+            '\x83\x72\x00\x03\x64\x00\x0D\x6E\x6F\x6E\x6F\x64\x65\x40\x6E' +
+            '\x6F\x68\x6F\x73\x74\x00\x00\x03\xE8\x4E\xE7\x68\x00\x02\xA4' +
+            '\xC8\x53\x40'
+        );
+        Erlang.binary_to_term(ref_new_binary,
+                              function(err, term) {
+            assert.strictEqual(err, undefined);
+            assert.ok(term instanceof Erlang.OtpErlangReference);
+            Erlang.term_to_binary(term,
+                                  function(err, binary) {
+                assert.strictEqual(err, undefined);
+                assert.equal(binary.toString('binary'),
+                    '\x83r\x00\x03s\rnonode@nohost\x00\x00\x03\xe8' +
+                    'N\xe7h\x00\x02\xa4\xc8S@');
+            });
+        });
+        var ref_newer_binary = (
+            '\x83\x5A\x00\x03\x64\x00\x0D\x6E\x6F\x6E\x6F\x64\x65\x40\x6E' +
+            '\x6F\x68\x6F\x73\x74\x00\x00\x00\x00\x00\x01\xAC\x03\xC7\x00' +
+            '\x00\x04\xBB\xB2\xCA\xEE'
+        );
+        Erlang.binary_to_term(ref_newer_binary,
+                              function(err, term) {
+            assert.strictEqual(err, undefined);
+            assert.ok(term instanceof Erlang.OtpErlangReference);
+            Erlang.term_to_binary(term,
+                                  function(err, binary) {
+                assert.strictEqual(err, undefined);
+                assert.equal(binary.toString('binary'),
+                    '\x83Z\x00\x03s\rnonode@nohost\x00\x00\x00\x00\x00' +
+                    '\x01\xac\x03\xc7\x00\x00\x04\xbb\xb2\xca\xee');
+            });
+        });
+    }).call(this);
     (function test_binary_to_term_compressed_term () {
         Erlang.binary_to_term('\x83P', function(err, term) {
             assert.ok(err instanceof Erlang.ParseException);
@@ -849,7 +952,7 @@ var hex = function hex(buffer) {
             assert.equal(binary.toString('binary'), '\x83k\0\4test');
         });
     }).call(this);
-    (function test_term_to_binary_boolean () {
+    (function test_term_to_binary_predefined_atom () {
         Erlang.term_to_binary(true, function(err, binary) {
             assert.strictEqual(err, undefined);
             assert.equal(binary.toString('binary'), '\x83s\4true');
@@ -858,6 +961,10 @@ var hex = function hex(buffer) {
             assert.strictEqual(err, undefined);
             assert.equal(binary.toString('binary'), '\x83s\5false');
         });
+        Erlang.term_to_binary(undefined, function(err, binary) {
+            assert.strictEqual(err, undefined);
+            assert.equal(binary.toString('binary'), '\x83s\x09undefined');
+        });
     }).call(this);
     (function test_term_to_binary_short_integer () {
         Erlang.term_to_binary(0, function(err, binary) {
