@@ -4,7 +4,7 @@
 #
 # MIT License
 #
-# Copyright (c) 2012-2018 Michael Truog <mjtruog at protonmail dot com>
+# Copyright (c) 2012-2019 Michael Truog <mjtruog at protonmail dot com>
 #
 # Permission is hereby granted, free of charge, to any person obtaining a
 # copy of this software and associated documentation files (the "Software"),
@@ -41,12 +41,17 @@ __all__ = [
     'InvalidInputException',
     'MessageDecodingException',
     'TerminateException',
-    # XXX backwards-compatibility
-    'invalid_input_exception',
-    'message_decoding_exception',
-    'terminate_exception',
 ]
 
+if sys.version_info[0] >= 3:
+    def _function_argc(function):
+        args, _, _, _, _, _, _ = inspect.getfullargspec(function)
+        return len(args)
+else:
+    def _function_argc(function):
+        args, _, _, _ = inspect.getargspec(function)
+        return len(args)
+
 # pylint: disable=too-many-instance-attributes
 # pylint: disable=too-many-public-methods
 class API(object):
@@ -80,8 +85,7 @@ class API(object):
         """
         subscribes to a service name pattern with a callback
         """
-        args, _, _, _ = inspect.getargspec(function)
-        if len(args) != 10:
+        if _function_argc(function) != 10:
             # self + arguments for a member function
             #  api + arguments for a static function
             raise InvalidInputException()
@@ -322,6 +326,15 @@ class API(object):
         except Exception as exception:
             self.__rethrow_exception(exception)
 
+    def shutdown(self, reason=None):
+        """
+        shutdown the service successfully
+        """
+        kwargs = {}
+        if reason is not None:
+            kwargs['reason'] = reason
+        return self.__api.shutdown(**kwargs)
+
     def __rethrow_exception(self, exception):
         if isinstance(exception, libcloudi_py.message_decoding_exception):
             raise MessageDecodingException(str(exception))
@@ -361,8 +374,6 @@ class InvalidInputException(Exception):
         if message is None:
             message = 'Invalid Input'
         Exception.__init__(self, message)
-# XXX backwards-compatibility
-invalid_input_exception = InvalidInputException
 
 class ReturnSyncException(Exception):
     """
@@ -398,8 +409,6 @@ class MessageDecodingException(Exception):
     """
     def __init__(self, message):
         Exception.__init__(self, message)
-# XXX backwards-compatibility
-message_decoding_exception = MessageDecodingException
 
 class TerminateException(Exception):
     """
@@ -414,8 +423,6 @@ class TerminateException(Exception):
         return the termination timeout
         """
         return self.__timeout
-# XXX backwards-compatibility
-terminate_exception = TerminateException
 
 # force unbuffered stdout/stderr handling without external configuration
 if sys.stderr.__class__.__name__ != '_unbuffered':
@@ -434,7 +441,8 @@ if sys.stderr.__class__.__name__ != '_unbuffered':
                 )
             else:
                 import codecs
-                self.__stream = codecs.getwriter('UTF-8')(stream)
+                self.encoding = 'UTF-8'
+                self.__stream = codecs.getwriter(self.encoding)(stream)
 
         def write(self, data):
             """
@@ -448,4 +456,3 @@ if sys.stderr.__class__.__name__ != '_unbuffered':
 
     sys.stdout = _unbuffered(sys.stdout)
     sys.stderr = _unbuffered(sys.stderr)
-
