@@ -94,12 +94,12 @@
 %% generate an error log message by cloudi_x_exometer.
 %%
 %%
-%% === cloudi_x_exometer_report/4 ===
+%% === cloudi_x_exometer_report/5 ===
 %%
 %% The `cloudi_x_exometer_report()' function is invoked as follows:
 %%
 %% <pre lang="erlang">
-%%      cloudi_x_exometer_report(Metric, DataPoint, State)</pre>
+%%      cloudi_x_exometer_report(Metric, DataPoint, Extra, Value, State)</pre>
 %%
 %% The custom plugin will receive this call when a periodic subscription
 %% triggers and wants to report its current value through the plugin.
@@ -111,6 +111,9 @@
 %% + `DataPoint'<br/>Specifies the data point or data points within the metric
 %%  to be reported.
 %%
+%% + `Extra'<br/>Specifies the extra data, which can be anything the reporter
+%% can understand.
+%%
 %% + `State'<br/>Contains the state returned by the last called plugin function.
 %%
 %% The `cloudi_x_exometer_report()' function should return `{ok, State}' where
@@ -140,6 +143,8 @@
 %% + `Extra'<br/>Specifies the extra data, which can be anything the reporter
 %% can understand.
 %%
+%% + `Value'<br/>Specifies the value for the datapoint, which is reported.
+%%
 %% + `State'<br/>Contains the state returned by the last called plugin function.
 %%
 %% The `exometer_unsubscribe()' function should return `{ok, State}' where
@@ -161,7 +166,7 @@
 %% That is, e.g. when a `select' pattern is used, all found values are passed
 %% to the reporter in one message. If bulk reporting is not enabled, each
 %% datapoint/value pair will be passed separately to the
-%% <a href="#cloudi_x_exometer_report/4"><code>cloudi_x_exometer_report/4</code></a> function. If `report_bulk' was enabled, the
+%% <a href="#cloudi_x_exometer_report/5"><code>cloudi_x_exometer_report/5</code></a> function. If `report_bulk' was enabled, the
 %% reporter callback will get all values at once. Note that this happens
 %% also for single values, which are then passed as a list of one metric,
 %% with a list of one datapoint/value pair.
@@ -276,7 +281,7 @@
           reporter              :: module()     | '_',
           metric                :: metric()     | '_',
           datapoint             :: datapoints() | '_',
-          retry_failed_metrics  :: boolean()    | '_',
+          retry_failed_metrics  :: boolean()    | undefined | '_',
           extra                 :: extra()      | '_'
          }).
 
@@ -302,7 +307,7 @@
 -record(reporter, {
           name      :: atom()                | '_',
           pid       :: pid()                 | atom(), % in select()
-          mref      :: reference()           | '_',
+          mref      :: reference()           | undefined | '_',
           module    :: module()              | '_',
           opts = [] :: [{atom(), any()}]     | '_',
           intervals = [] :: [#interval{}]    | '_',
@@ -375,12 +380,14 @@ subscribe(Reporter, Metric, DataPoint, Interval, Extra, Retry)
                           retry_failed_metrics = Retry,
                           extra = Extra}, Interval}).
 
+-dialyzer({no_return, unsubscribe/3}).
 -spec unsubscribe(module(), metric(), datapoint()) ->
                          ok | not_found.
 %% @equiv unsubscribe(Reporter, Metric, DataPoint, [])
 unsubscribe(Reporter, Metric, DataPoint) ->
     unsubscribe(Reporter, Metric, DataPoint, []).
 
+-dialyzer({no_return, unsubscribe/4}).
 -spec unsubscribe(module(), metric(), datapoint() | [datapoint()], extra()) ->
                          ok | not_found.
 %% @doc Removes a subscription.
@@ -654,6 +661,26 @@ new_entry(Entry) ->
 %%--------------------------------------------------------------------
 init([]) ->
     process_flag(trap_exit, true),
+    D = ets:foldl(
+        fun (#reporter{name = Name, module = Module, restart = Restart} = R, Acc) ->
+                terminate_reporter(R),
+                case add_restart(Restart) of
+                    {remove, How} ->
+                        case How of
+                            {M, F} when is_atom(M), is_atom(F) ->
+                                try M:F(Module, {?MODULE, parent_restart}) catch _:_ -> ok end;
+                            _ ->
+                                ok
+                        end,
+                        [Name | Acc];
+                    {restart, Restart1} ->
+                        restart_reporter(R#reporter{restart = Restart1}),
+                        Acc
+                end
+        end,
+        [],
+        ?EXOMETER_REPORTERS),
+    [ets:delete(?EXOMETER_REPORTERS, R) || R <- D],
     {ok, #st{}}.
 
 start_reporters() ->
@@ -700,6 +727,7 @@ do_start_reporters(S) ->
     end,
     S#st{}.
 
+-spec make_reporter_recs([{atom(), list()}]) -> [#reporter{}].
 make_reporter_recs([{R, Opts}|T]) when is_atom(R), is_list(Opts) ->
     [#reporter{name = R,
                module = get_module(R, Opts),
@@ -817,15 +845,14 @@ handle_call({subscribe,
                    retry_failed_metrics = RetryFailedMetrics,
                    extra = Extra} , Interval },
             _From, #st{} = St) ->
-
     %% Verify that the given metric/data point actually exist.
     case ets:lookup(?EXOMETER_REPORTERS, Reporter) of
-        [#reporter{status = Status}] ->
+        [#reporter{status = Status, pid=ReporterPid}] ->
             case is_valid_metric(Metric, DataPoint) of
                 true ->
                     if Status =:= enabled ->
-                            Reporter ! {exometer_subscribe, Metric,
-                                        DataPoint, Interval, Extra};
+                            ReporterPid ! {exometer_subscribe, Metric,
+                                           DataPoint, Interval, Extra};
                        true -> ignore
                     end,
                     subscribe_(Reporter, Metric, DataPoint,
@@ -1148,7 +1175,20 @@ maybe_enable_subscriptions(#cloudi_x_exometer_entry{name = Metric}) ->
       end, ets:select(?EXOMETER_SUBS,
                       [{#subscriber{key = #key{metric = Metric,
                                                _ = '_'},
-                                    _ = '_'}, [], ['$_']}])).
+                                    _ = '_'}, [], ['$_']}])),
+    %% Also re-check the static subscribers for select and apply
+    case lists:keyfind(subscribers, 1, get_report_env()) of
+        {subscribers, Subscribers} ->
+            lists:foreach(
+                fun(Sub) ->
+                    case Sub of
+                        {select, _} -> init_subscriber(Sub);
+                        {apply, _} -> init_subscriber(Sub);
+                        _ -> ok
+                    end
+                end, Subscribers);
+        false -> []
+    end.
 
 resubscribe(#subscriber{key = #key{reporter = RName,
                                    metric = Metric,
@@ -1233,8 +1273,9 @@ cancel_subscr_timers(Reporter) ->
                                     _ = '_'}, [], ['$_']}])).
 
 restart_subscr_timer(Key, Interval, T0) when is_integer(Interval) ->
-    TRef = erlang:send_after(adjust_interval(Interval, T0), self(),
-                             subscr_timer_msg(Key, Interval, T0)),
+    {AdjInt, RptTime} = adjust_interval(Interval, T0),
+    TRef = erlang:send_after(AdjInt, self(),
+                             subscr_timer_msg(Key, Interval, RptTime)),
     ets:update_element(?EXOMETER_SUBS, Key,
                        [{#subscriber.t_ref, TRef}]);
 restart_subscr_timer(_, _, _) ->
@@ -1245,9 +1286,10 @@ restart_batch_timer(Name, #reporter{name = Reporter,
     case lists:keyfind(Name, #interval.name, Ints) of
         #interval{time = Time, t_ref = OldTRef} = I when is_integer(Time) ->
             cancel_timer(OldTRef),
-            TRef = erlang:send_after(
-                     adjust_interval(Time, T0), self(),
-                     batch_timer_msg(Reporter, Name, Time, T0)),
+            {Int, RptTime} = adjust_interval(Time, T0),
+            TRef = erlang:send_after(Int, self(),
+                                     batch_timer_msg(
+                                       Reporter, Name, Time, RptTime)),
             ets:update_element(?EXOMETER_REPORTERS, Reporter,
                                [{#reporter.intervals,
                                  lists:keyreplace(Name, #interval.name, Ints,
@@ -1260,7 +1302,13 @@ restart_batch_timer(Name, #reporter{name = Reporter,
 
 adjust_interval(Time, T0) ->
     T1 = os:timestamp(),
-    erlang:max(0, Time - tdiff(T1, T0)).
+    case tdiff(T1, T0) of
+        D when D > Time; D < 0 ->
+            %% Most likely due to clock adjustment
+            {Time, T1};
+        D ->
+            {Time-D, T0}
+    end.
 
 tdiff(T1, T0) ->
     timer:now_diff(T1, T0) div 1000.
@@ -1460,16 +1508,21 @@ terminate_reporter(#reporter{pid = undefined}) ->
 
 subscribe_(Reporter, Metric, DataPoint, Interval, RetryFailedMetrics,
            Extra, Status) ->
+    ?log(debug, "subscribe_(~p, ~p, ~p, ~p, ~p, ~p, ~p)~n", [Reporter, Metric, DataPoint, Interval, RetryFailedMetrics, Extra, Status]),
     Key = #key{reporter = Reporter,
                metric = Metric,
                datapoint = DataPoint,
                extra = Extra,
                retry_failed_metrics = RetryFailedMetrics
               },
-    ets:insert(?EXOMETER_SUBS,
-               #subscriber{key = Key,
-                           interval = Interval,
-                           t_ref = maybe_send_after(Status, Key, Interval)}).
+    case ets:lookup(?EXOMETER_SUBS, Key) of
+        [] -> ets:insert(?EXOMETER_SUBS,
+                 #subscriber{key = Key,
+                             interval = Interval,
+                             t_ref = maybe_send_after(Status, Key, Interval)});
+        _ ->
+            ?log(debug, "subscribe_(): not adding duplicate subscription")
+        end.
 
 maybe_send_after(enabled, Key, Interval) when is_integer(Interval) ->
     erlang:send_after(
@@ -1477,6 +1530,7 @@ maybe_send_after(enabled, Key, Interval) when is_integer(Interval) ->
 maybe_send_after(_, _, _) ->
     undefined.
 
+-dialyzer({no_return, unsubscribe_/4}).
 unsubscribe_(Reporter, Metric, DataPoint, Extra) ->
     ?log(info, "unsubscribe_(~p, ~p, ~p, ~p)~n",
           [ Reporter, Metric, DataPoint, Extra]),
@@ -1494,19 +1548,17 @@ unsubscribe_(#subscriber{key = #key{reporter = Reporter,
                                     metric = Metric,
                                     datapoint = DataPoint,
                                     extra = Extra} = Key, t_ref = TRef}) ->
-    try_send(
-      Reporter, {exometer_unsubscribe, Metric, DataPoint, Extra}),
+    try_send(Reporter, {exometer_unsubscribe, Metric, DataPoint, Extra}),
     cancel_timer(TRef),
     ets:delete(?EXOMETER_SUBS, Key),
     ok.
 
-
 report_values(Found, #key{reporter = Reporter, extra = Extra} = Key) ->
     try Reporter ! {cloudi_x_exometer_report, Found, Extra}
     catch
-        error:Reason ->
+        ?EXCEPTION(error, Reason, Stacktrace) ->
             ?log(error, "~p~nKey = ~p~nTrace: ~p",
-                        [Reason, Key, erlang:get_stacktrace()])
+                        [Reason, Key, ?GET_STACK(Stacktrace)])
     end.
 
 retrieve_metric({Metric, Type, Enabled}, Acc) ->
