@@ -16,10 +16,13 @@
   checkout/4,
   checkin/2]).
 
--export([start_pool/2,
+-export([
+  get_stats/1,
+  start_pool/2,
   stop_pool/1,
   find_pool/1,
-  notify/2]).
+  notify/2
+]).
 
 
 -export([count/1, count/2,
@@ -45,7 +48,8 @@
   max_connections,
   timeout,
   clients = dict:new(),
-  queues = dict:new(),  % Dest => queue of Froms
+  queues = dict:new(),  % Dest => queue of Froms,
+  pending = dict:new(),
   connections = dict:new(),
   sockets = dict:new(),
   nb_waiters=0}).
@@ -64,8 +68,10 @@ checkout(Host0, Port, Transport, #client{options=Opts}=Client) ->
   Name = proplists:get_value(pool, Opts, default),
   Pool = find_pool(Name, Opts),
   ConnectTimeout = proplists:get_value(connect_timeout, Opts, 8000),
+  %% Fall back to using connect_timeout if checkout_timeout is not set
+  CheckoutTimeout = proplists:get_value(checkout_timeout, Opts, ConnectTimeout),
   case catch gen_server:call(Pool, {checkout, {Host, Port, Transport},
-    Pid, RequestRef}, ConnectTimeout) of
+    Pid, RequestRef}, CheckoutTimeout) of
     {ok, Socket, Owner} ->
       CheckinReference = {Host, Port, Transport},
       {ok, {Name, RequestRef, CheckinReference, Owner, Transport}, Socket};
@@ -75,7 +81,10 @@ checkout(Host0, Port, Transport, #client{options=Opts}=Client) ->
         Transport}};
     {error, Reason} ->
       {error, Reason};
-    {'EXIT', {timeout, _}} -> {error, connect_timeout}
+    {'EXIT', {timeout, _}} ->
+      % socket will still checkout so to avoid deadlock we send in a cancellation
+      gen_server:cast(Pool, {checkout_cancel, {Host, Port, Transport}, RequestRef}),
+      {error, checkout_timeout}
   end.
 
 %% @doc release a socket in the pool
@@ -96,6 +105,9 @@ checkin({_Name, Ref, Dest, Owner, Transport}, Socket) ->
       ok
   end.
 
+get_stats(Pool) ->
+  gen_server:call(find_pool(Pool), stats).
+
 
 %% @doc start a pool
 start_pool(Name, Options) ->
@@ -206,7 +218,6 @@ start_link(Name, Options0) ->
   gen_server:start_link(?MODULE, [Name, Options], []).
 
 init([Name, Options]) ->
-  process_flag(priority, high),
   case lists:member({seed,1}, ssl:module_info(exports)) of
     true ->
       % Make sure that the ssl random number generator is seeded
@@ -233,6 +244,8 @@ init([Name, Options]) ->
   {ok, #state{name=Name, cloudi_x_metrics=Engine, max_connections=MaxConn,
     timeout=Timeout}}.
 
+handle_call(stats, _From, State) ->
+  {reply, handle_stats(State), State};
 handle_call(count, _From, #state{sockets=Sockets}=State) ->
   {reply, dict:size(Sockets), State};
 handle_call(timeout, _From, #state{timeout=Timeout}=State) ->
@@ -245,7 +258,7 @@ handle_call({checkout, Dest, Pid, RequestRef}, From, State) ->
     max_connections=MaxConn,
     clients=Clients,
     queues = Queues,
-    nb_waiters = NbWaiters} = State,
+    pending = Pending} = State,
 
   {Reply, State2} = find_connection(Dest, Pid, State),
   case Reply of
@@ -257,12 +270,11 @@ handle_call({checkout, Dest, Pid, RequestRef}, From, State) ->
       case dict:size(Clients) >= MaxConn of
         true ->
           Queues2 = add_to_queue(Dest, From, RequestRef, Queues),
-          NbWaiters2 = NbWaiters + 1,
-          _ = cloudi_x_metrics:update_histogram(Engine,
-            [cloudi_x_hackney_pool, PoolName, queue_count],
-            NbWaiters2),
-          {noreply, State2#state{queues = Queues2,
-            nb_waiters=NbWaiters2}};
+          Pending2 =add_pending(RequestRef, From, Dest, Pending),
+          _ = cloudi_x_metrics:update_histogram(
+            Engine, [cloudi_x_hackney_pool, PoolName, queue_count], dict:size(Pending2)
+          ),
+          {noreply, State2#state{queues = Queues2, pending = Pending2}};
         false ->
           State3 = monitor_client(Dest, RequestRef, State2),
           ok = update_usage(State3),
@@ -304,6 +316,17 @@ handle_cast({set_maxconn, MaxConn}, State) ->
 handle_cast({set_timeout, NewTimeout}, State) ->
   {noreply, State#state{timeout=NewTimeout}};
 
+handle_cast({checkout_cancel, Dest, Ref}, State) ->
+  #state{queues=Queues, pending=Pending} = State,
+  {Queues2, Removed} = del_from_queue(Dest, Ref, Queues),
+  case Removed of
+    true ->
+      Pending2 = del_pending(Ref, Pending),
+      {noreply, State#state{queues=Queues2, nb_waiters=Pending2}};
+    false ->
+      % we leak the socket here but 'DOWN' will mop up for us when it times out
+      {noreply, dequeue(Dest, Ref, State)}
+  end;
 handle_cast(_Msg, State) ->
   {noreply, State}.
 
@@ -324,21 +347,10 @@ handle_info({ssl_error, Socket, _}, State) ->
 handle_info({'DOWN', Ref, request, _Pid, _Reason}, State) ->
   case dict:find(Ref, State#state.clients) of
     {ok, Dest} ->
-      Clients2 = dict:erase(Ref, State#state.clients),
-      case queue_out(Dest, State#state.queues) of
-        empty ->
-          {noreply, State#state{clients = Clients2}};
-        {ok, {From, Ref2}, Queues2} ->
-          NbWaiters = State#state.nb_waiters - 1,
-          _ = cloudi_x_metrics:update_histogram(State#state.cloudi_x_metrics,
-            [cloudi_x_hackney_pool, State#state.name, queue_count], NbWaiters),
-          gen_server:reply(From, {error, no_socket, self()}),
-          State2 = State#state{queues = Queues2, clients = Clients2,
-            nb_waiters=NbWaiters},
-          {noreply, monitor_client(Dest, Ref2, State2)}
-      end;
+      {noreply, dequeue(Dest, Ref, State)};
     error ->
-      {noreply, State}
+      NewState = remove_pending(Ref, State),
+      {noreply, NewState}
   end;
 handle_info(_, State) ->
   {noreply, State}.
@@ -359,6 +371,22 @@ terminate(_Reason, #state{name=PoolName, cloudi_x_metrics=Engine, sockets=Socket
 
 %% internals
 
+dequeue(Dest, Ref, State) ->
+  #state{clients=Clients, queues=Queues, pending=Pending} = State,
+  Clients2 = dict:erase(Ref, Clients),
+  case queue_out(Dest, Queues) of
+    empty ->
+      State#state{clients = Clients2};
+    {ok, {From, Ref2}, Queues2} ->
+      Pending2 = del_pending(Ref, Pending),
+      _ = cloudi_x_metrics:update_histogram(
+        State#state.cloudi_x_metrics, [cloudi_x_hackney_pool, State#state.name, queue_count], dict:size(Pending2)
+      ),
+      gen_server:reply(From, {error, no_socket, self()}),
+      State2 = State#state{queues = Queues2, clients = Clients2, pending=Pending2},
+      monitor_client(Dest, Ref2, State2)
+  end.
+
 find_connection({_Host, _Port, Transport}=Dest, Pid,
   #state{connections=Conns, sockets=Sockets}=State) ->
   case dict:find(Dest, Conns) of
@@ -372,8 +400,7 @@ find_connection({_Host, _Port, Transport}=Dest, Pid,
               cancel_timer(S, Timer),
               NewConns = update_connections(Rest, Dest, Conns),
               NewSockets = dict:erase(S, Sockets),
-              NewState = State#state{connections=NewConns,
-                sockets=NewSockets},
+              NewState = State#state{connections=NewConns, sockets=NewSockets},
               {{ok, S, self()}, NewState};
             {error, badarg} ->
               %% something happened here normally the PID died,
@@ -453,6 +480,25 @@ add_to_queue({_Host, _Port, _Transport} = Dest, From, Ref, Queues) ->
       dict:store(Dest, queue:in({From, Ref}, Q), Queues)
   end.
 
+%------------------------------------------------------------------------------
+%% @private
+%%------------------------------------------------------------------------------
+del_from_queue({_Host, _Port, _Transport} = Dest, Ref, Queues) ->
+  case dict:find(Dest, Queues) of
+    error ->
+      {Queues, false};
+    {ok, Q} ->
+      Q2 = queue:filter(fun({_, R}) -> R =/= Ref end, Q),
+      Removed = queue:len(Q) =/= queue:len(Q2),
+      Queues2 = case queue:is_empty(Q2) of
+                  true ->
+                    dict:erase(Dest, Queues);
+                  false ->
+                    dict:store(Dest, Q2, Queues)
+                end,
+      {Queues2, Removed}
+  end.
+
 %------------------------------------------------------------------------------
 %% @private
 %%------------------------------------------------------------------------------
@@ -475,26 +521,25 @@ queue_out({_Host, _Port, _Transport} = Dest, Queues) ->
 %% @private
 %%------------------------------------------------------------------------------
 deliver_socket(Socket, {_, _, Transport} = Dest, State) ->
-  case queue_out(Dest, State#state.queues) of
+  #state{queues = Queues, pending=Pending} = State,
+  case queue_out(Dest, Queues) of
     empty ->
       store_socket(Dest, Socket, State);
     {ok, {{PidWaiter, _} = FromWaiter, Ref}, Queues2} ->
-      NbWaiters = State#state.nb_waiters - 1,
-      _ = cloudi_x_metrics:update_histogram(State#state.cloudi_x_metrics,
-        [cloudi_x_hackney_pool, State#state.name, queue_count],
-        NbWaiters),
+      Pending2 = del_pending(Ref, Pending),
+      _ = cloudi_x_metrics:update_histogram(
+        State#state.cloudi_x_metrics, [cloudi_x_hackney_pool, State#state.name, queue_count], dict:size(Pending2)
+      ),
       case Transport:controlling_process(Socket, PidWaiter) of
         ok ->
           gen_server:reply(FromWaiter, {ok, Socket, self()}),
-          monitor_client(Dest, Ref,
-            State#state{queues = Queues2,
-              nb_waiters=NbWaiters});
+          monitor_client(Dest, Ref, State#state{queues = Queues2, pending=Pending2});
         _Error ->
           % Something wrong, close the socket
-            catch Transport:close(Socket),
+          _ = (catch Transport:close(Socket)),
           %% and let the waiter connect to a new one
           gen_server:reply(FromWaiter, {error, no_socket, self()}),
-          State#state{queues = Queues2, nb_waiters = NbWaiters}
+          State#state{queues = Queues2, pending = Pending2}
       end
   end.
 
@@ -510,6 +555,37 @@ sync_socket(Transport, Socket) ->
     true
   end.
 
+
+add_pending(Ref, From, Dest, Pending) ->
+  dict:store(Ref, {From, Dest}, Pending).
+
+del_pending(Ref, Pending) ->
+  dict:erase(Ref, Pending).
+
+remove_pending(Ref, #state{queues=Queues0, pending=Pending0} = State) ->
+  case dict:find(Ref, Pending0) of
+    {ok, {From, Dest}} ->
+      Pending1 = dict:erase(Ref, Pending0),
+      Queues1 = case dict:find(Dest, Queues0) of
+                  {ok, Q0} ->
+                    Q1 = queue:filter(
+                      fun
+                        (PendingReq) when PendingReq =:= {From, Ref} -> false;
+                        (_) -> true
+                      end,
+                      Q0
+                    ),
+                    dict:store(Dest, Q1, Queues0);
+                  error ->
+                    Queues0
+                end,
+      State#state{queues=Queues1, pending=Pending1};
+    error ->
+      State
+  end.
+
+
+
 %------------------------------------------------------------------------------
 %% @private
 %%------------------------------------------------------------------------------
@@ -520,7 +596,7 @@ monitor_client(Dest, Ref, State) ->
 
 init_metrics(PoolName) ->
   %% get cloudi_x_metrics module
-  Engine = cloudi_x_metrics:init(cloudi_x_hackney_util:mod_metrics()),
+  Engine = cloudi_x_hackney_metrics:get_engine(),
 
   %% initialise cloudi_x_metrics
   _ = cloudi_x_metrics:new(Engine, histogram, [cloudi_x_hackney_pool, PoolName, take_rate]),
@@ -547,3 +623,12 @@ update_usage(
   _ = cloudi_x_metrics:update_histogram(Engine, [cloudi_x_hackney_pool, PoolName, free_count],
     dict:size(Sockets) - 1),
   ok.
+
+
+handle_stats(State) ->
+  #state{name=PoolName, max_connections=Max, sockets=Sockets, clients=Clients, pending=Pending} = State,
+  [{name, PoolName},
+   {max, Max},
+   {in_use_count,  dict:size(Clients)},
+   {free_count, dict:size(Sockets)},
+   {queue_count, dict:size(Pending)}].
