@@ -9,7 +9,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2011-2018 Michael Truog <mjtruog at protonmail dot com>
+%%% Copyright (c) 2011-2019 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -30,8 +30,8 @@
 %%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog at protonmail dot com>
-%%% @copyright 2011-2018 Michael Truog
-%%% @version 1.7.4 {@date} {@time}
+%%% @copyright 2011-2019 Michael Truog
+%%% @version 1.8.0 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_service_api).
@@ -55,6 +55,7 @@
          nodes_remove/2,
          nodes_alive/1,
          nodes_dead/1,
+         nodes_status/2,
          nodes/1,
          logging_set/2,
          logging_file_set/2,
@@ -209,7 +210,7 @@
               aspect_init_after_internal/0,
               aspect_init_after_external/0]).
 -type aspect_request_before_internal_f() ::
-    fun((Type :: cloudi_service:request_type(),
+    fun((RequestType :: cloudi_service:request_type(),
          Name :: cloudi_service:service_name(),
          Pattern :: cloudi_service:service_name_pattern(),
          RequestInfo :: cloudi_service:request_info(),
@@ -223,7 +224,7 @@
         {ok, NewState :: any()} |
         {stop, Reason :: any(), NewState :: any()}).
 -type aspect_request_before_external_f() ::
-    fun((Type :: cloudi_service:request_type(),
+    fun((RequestType :: cloudi_service:request_type(),
          Name :: cloudi_service:service_name(),
          Pattern :: cloudi_service:service_name_pattern(),
          RequestInfo :: cloudi_service:request_info(),
@@ -236,7 +237,7 @@
         {ok, NewState :: any()} |
         {stop, Reason :: any(), NewState :: any()}).
 -type aspect_request_after_internal_f() ::
-    fun((Type :: cloudi_service:request_type(),
+    fun((RequestType :: cloudi_service:request_type(),
          Name :: cloudi_service:service_name(),
          Pattern :: cloudi_service:service_name_pattern(),
          RequestInfo :: cloudi_service:request_info(),
@@ -251,7 +252,7 @@
         {ok, NewState :: any()} |
         {stop, Reason :: any(), NewState :: any()}).
 -type aspect_request_after_external_f() ::
-    fun((Type :: cloudi_service:request_type(),
+    fun((RequestType :: cloudi_service:request_type(),
          Name :: cloudi_service:service_name(),
          Pattern :: cloudi_service:service_name_pattern(),
          RequestInfo :: cloudi_service:request_info(),
@@ -550,7 +551,10 @@
                             service_options_external()}).
 -type service() :: #internal{} | #external{}.
 -type service_status_internal() ::
-    nonempty_list({count_process, pos_integer()} |
+    nonempty_list({type, internal} |
+                  {prefix, cloudi:service_name_pattern()} |
+                  {module, atom()} |
+                  {count_process, pos_integer()} |
                   {uptime_total, nonempty_string()} |
                   {uptime_running, nonempty_string()} |
                   {uptime_restarts, nonempty_string()} |
@@ -575,7 +579,10 @@
                   {availability_year_running, nonempty_string()} |
                   {availability_year_updated, nonempty_string()}).
 -type service_status_external() ::
-    nonempty_list({count_process, pos_integer()} |
+    nonempty_list({type, external} |
+                  {prefix, cloudi:service_name_pattern()} |
+                  {file_path, file:filename()} |
+                  {count_process, pos_integer()} |
                   {count_thread, pos_integer()} |
                   {uptime_total, nonempty_string()} |
                   {uptime_running, nonempty_string()} |
@@ -790,8 +797,38 @@
                                          {ec2_host, string()} |
                                          {groups, list(string())} |
                                          {tags, list({string(), string()} |
-                                                     string())})})}).
--export_type([nodes_proplist/0]).
+                                                     string())})})} |
+                  {cost, list({node() | default, float()})} |
+                  {cost_precision, 0..253} |
+                  {log_reconnect, loglevel()}).
+-type node_status() ::
+    nonempty_list(% local node
+                  {uptime, nonempty_string()} |
+                  {uptime_cost_total, nonempty_string()} |
+                  {uptime_cost_day, nonempty_string()} |
+                  {uptime_cost_week, nonempty_string()} |
+                  {uptime_cost_month, nonempty_string()} |
+                  {uptime_cost_year, nonempty_string()} |
+                  % remote node
+                  {tracked, nonempty_string()} |
+                  {tracked_cost_total, nonempty_string()} |
+                  {tracked_cost_day, nonempty_string()} |
+                  {tracked_cost_week, nonempty_string()} |
+                  {tracked_cost_month, nonempty_string()} |
+                  {tracked_cost_year, nonempty_string()} |
+                  {tracked_disconnects, nonempty_string()} |
+                  {disconnected, boolean()} |
+                  {downtime_day_disconnected, nonempty_string()} |
+                  {downtime_week_disconnected, nonempty_string()} |
+                  {downtime_month_disconnected, nonempty_string()} |
+                  {downtime_year_disconnected, nonempty_string()} |
+                  % all nodes
+                  {availability_day, nonempty_string()} |
+                  {availability_week, nonempty_string()} |
+                  {availability_month, nonempty_string()} |
+                  {availability_year, nonempty_string()}).
+-export_type([nodes_proplist/0,
+              node_status/0]).
 
 -type aspect_log_f() ::
     fun((Level :: loglevel_on(),
@@ -995,7 +1032,7 @@ acl(Timeout)
 %% When a subscription on the same service name pattern occurred
 %% multiple times, only a single entry is returned within the list.
 %% Service name patterns that are subscriptions of non-service Erlang pids
-%% (e.g., cloudi_service_http_cowboy websocket connection pids) will not
+%% (e.g., cloudi_service_http_cowboy1 websocket connection pids) will not
 %% be returned by this function.
 %% @end
 %%-------------------------------------------------------------------------
@@ -1119,7 +1156,7 @@ services_restart([_ | _] = L, Timeout)
 %% Multiple services may be returned for a single service name.  Only service
 %% instances on the local Erlang node are searched.  Service names that match
 %% subscriptions of non-service Erlang pids only
-%% (e.g., cloudi_service_http_cowboy websocket connection pids) will not
+%% (e.g., cloudi_service_http_cowboy1 websocket connection pids) will not
 %% return the service's configuration with this function.  Provide a scope
 %% within a 2 element tuple with the service name to check a custom scope.
 %% @end
@@ -1146,7 +1183,7 @@ services_search(Name, Timeout)
         [_ | _] = ServiceNameValue ->
             {?SCOPE_DEFAULT, ServiceNameValue}
     end,
-    try cloudi_x_trie:is_pattern(ServiceName) of
+    try cloudi_x_trie:is_pattern2(ServiceName) of
         false ->
             cloudi_core_i_configurator:services_search(Scope, ServiceName,
                                                        Timeout);
@@ -1356,6 +1393,27 @@ nodes_dead(Timeout)
           (Timeout =:= infinity)) ->
     cloudi_core_i_nodes:dead(Timeout).
 
+%%-------------------------------------------------------------------------
+%% @doc
+%% ===List the current status of specific nodes.===
+%% @end
+%%-------------------------------------------------------------------------
+
+-spec nodes_status(L :: list(node()),
+                   Timeout :: api_timeout_milliseconds()) ->
+    {ok, nonempty_list({node(), node_status()})} |
+    {error,
+     timeout | noproc |
+     {node_not_found, any()}}.
+
+nodes_status(L, Timeout)
+    when is_list(L),
+         ((is_integer(Timeout) andalso
+           (Timeout > ?TIMEOUT_DELTA) andalso
+           (Timeout =< ?TIMEOUT_MAX_ERLANG)) orelse
+          (Timeout =:= infinity)) ->
+    cloudi_core_i_nodes:status(L, Timeout).
+
 %%-------------------------------------------------------------------------
 %% @doc
 %% ===List both the connected and disconnected CloudI nodes.===
