@@ -10,7 +10,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2012-2017 Michael Truog <mjtruog at gmail dot com>
+%%% Copyright (c) 2012-2019 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -30,13 +30,13 @@
 %%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 %%% DEALINGS IN THE SOFTWARE.
 %%%
-%%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2012-2017 Michael Truog
-%%% @version 1.7.3 {@date} {@time}
+%%% @author Michael Truog <mjtruog at protonmail dot com>
+%%% @copyright 2012-2019 Michael Truog
+%%% @version 1.8.0 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_service_map_reduce).
--author('mjtruog [at] gmail (dot) com').
+-author('mjtruog at protonmail dot com').
 
 -behaviour(cloudi_service).
 
@@ -54,7 +54,7 @@
 -define(DEFAULT_CONCURRENCY,                 1.0). % schedulers multiplier
 -define(DEFAULT_LOG_EXECUTION_TIME,         true).
 
--type map_send_args() :: list().
+-type map_send_args() :: nonempty_list().
 -export_type([map_send_args/0]).
 
 -record(state,
@@ -67,8 +67,9 @@
         map_requests :: #{cloudi_service:trans_id() := map_send_args()}
     }).
 
--record(init,
+-record(init_begin,
     {
+        service :: pid(),
         prefix :: string(),
         timeout :: cloudi_service_api:timeout_initialize_value_milliseconds(),
         time_start :: cloudi_timestamp:seconds_monotonic(),
@@ -78,6 +79,14 @@
         concurrency :: number()
     }).
 
+-record(init_end,
+    {
+        state :: undefined | #state{},
+        error = undefined :: any()
+    }).
+
+-define(INFO_RETRY_INTERVAL, 500). % milliseconds
+
 %%%------------------------------------------------------------------------
 %%% External interface functions
 %%%------------------------------------------------------------------------
@@ -153,55 +162,60 @@ cloudi_service_init(Args, Prefix, Timeout, _Dispatcher) ->
             ok = cloudi_x_reltool_util:module_loaded(MapReduceModule)
     end,
     % cloudi_service_init/4 is always executed by the service process
-    self() !  #init{prefix = Prefix,
-                    timeout = Timeout,
-                    time_start = TimeStart,
-                    log_execution_time = LogExecutionTime,
-                    map_reduce_module = MapReduceModule,
-                    map_reduce_args = MapReduceArgs,
-                    concurrency = Concurrency},
+    Service = self(),
+    Service !  #init_begin{service = Service,
+                           prefix = Prefix,
+                           timeout = Timeout,
+                           time_start = TimeStart,
+                           log_execution_time = LogExecutionTime,
+                           map_reduce_module = MapReduceModule,
+                           map_reduce_args = MapReduceArgs,
+                           concurrency = Concurrency},
     {ok, undefined}.
 
-cloudi_service_handle_request(_Type, _Name, _Pattern, _RequestInfo, _Request,
+cloudi_service_handle_request(_RequestType, _Name, _Pattern,
+                              _RequestInfo, _Request,
                               _Timeout, _Priority, _TransId, _Pid,
                               State, _Dispatcher) ->
     {reply, <<>>, State}.
 
-cloudi_service_handle_info(#init{prefix = Prefix,
-                                 timeout = Timeout,
-                                 time_start = TimeStart,
-                                 log_execution_time = LogExecutionTime,
-                                 map_reduce_module = MapReduceModule,
-                                 map_reduce_args = MapReduceArgs,
-                                 concurrency = Concurrency},
+cloudi_service_handle_info(#init_begin{service = Service} = InitBegin,
                            undefined, Dispatcher) ->
-    % cloudi_service_map_reduce_new/3 execution occurs outside of
-    % cloudi_service_init/3 to allow send_sync and recv_sync function calls
+    % cloudi_service_map_reduce_new/5 execution occurs outside of
+    % cloudi_service_init/4 to allow send_sync and recv_async function calls
     % because no Erlang process linking/spawning/etc. should be occurring,
-    % only algorithmic initialization
-    MapCount = cloudi_concurrency:count(Concurrency),
-    case MapReduceModule:cloudi_service_map_reduce_new(MapReduceArgs,
-                                                       MapCount,
-                                                       Prefix,
-                                                       Timeout,
-                                                       Dispatcher) of
-        {ok, MapReduceState} ->
-            case map_send(MapCount, #{}, Dispatcher,
-                          MapReduceModule, MapReduceState) of
-                {ok, MapRequests, NewMapReduceState} ->
-                    {noreply, #state{time_start = TimeStart,
-                                     log_execution_time = LogExecutionTime,
-                                     map_reduce_module = MapReduceModule,
-                                     map_reduce_state = NewMapReduceState,
-                                     map_count = MapCount,
-                                     map_requests = MapRequests}};
-                {error, _} = Error ->
-                    {stop, Error, undefined}
-            end;
-        {error, _} = Error ->
-            {stop, Error, undefined}
+    % only algorithmic initialization.  Initialization is done in a temporary
+    % process so there is no blocking problem (with send_sync or recv_async)
+    % if duo_mode == true.  No timeout is enforced on
+    % cloudi_service_map_reduce_new/5 execution.
+    _ = erlang:spawn_link(fun() ->
+        case init(InitBegin, Dispatcher) of
+            {noreply, State} ->
+                Service ! #init_end{state = State};
+            {stop, Reason, State} when Reason /= undefined ->
+                Service ! #init_end{state = State,
+                                    error = Reason}
+        end,
+        true = erlang:unlink(Service)
+    end),
+    {noreply, undefined};
+
+cloudi_service_handle_info(#init_end{state = State,
+                                     error = Error},
+                           undefined, _Dispatcher) ->
+    if
+        Error =:= undefined ->
+            {noreply, State};
+        true ->
+            {stop, Error, State}
     end;
 
+cloudi_service_handle_info(Request, undefined, Dispatcher) ->
+    % waiting for #init_end{} still
+    erlang:send_after(?INFO_RETRY_INTERVAL,
+                      cloudi_service:self(Dispatcher), Request),
+    {noreply, undefined};
+
 cloudi_service_handle_info(#timeout_async_active{trans_id = TransId} = Request,
                            #state{map_reduce_module = MapReduceModule,
                                   map_reduce_state = MapReduceState,
@@ -293,6 +307,37 @@ cloudi_service_terminate(_Reason, _Timeout, #state{}) ->
 %%% Private functions
 %%%------------------------------------------------------------------------
 
+init(#init_begin{prefix = Prefix,
+                 timeout = Timeout,
+                 time_start = TimeStart,
+                 log_execution_time = LogExecutionTime,
+                 map_reduce_module = MapReduceModule,
+                 map_reduce_args = MapReduceArgs,
+                 concurrency = Concurrency},
+     Dispatcher) ->
+    MapCount = cloudi_concurrency:count(Concurrency),
+    case MapReduceModule:cloudi_service_map_reduce_new(MapReduceArgs,
+                                                       MapCount,
+                                                       Prefix,
+                                                       Timeout,
+                                                       Dispatcher) of
+        {ok, MapReduceState} ->
+            case map_send(MapCount, #{}, Dispatcher,
+                          MapReduceModule, MapReduceState) of
+                {ok, MapRequests, NewMapReduceState} ->
+                    {noreply, #state{time_start = TimeStart,
+                                     log_execution_time = LogExecutionTime,
+                                     map_reduce_module = MapReduceModule,
+                                     map_reduce_state = NewMapReduceState,
+                                     map_count = MapCount,
+                                     map_requests = MapRequests}};
+                {error, _} = Error ->
+                    {stop, Error, undefined}
+            end;
+        {error, _} = Error ->
+            {stop, Error, undefined}
+    end.
+
 map_send(MapRequests, Dispatcher, MapReduceModule, MapReduceState) ->
     map_send(1, MapRequests, Dispatcher, MapReduceModule, MapReduceState).
 
