@@ -47,7 +47,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2015-2017 Michael Truog <mjtruog at gmail dot com>
+%%% Copyright (c) 2015-2019 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -67,13 +67,13 @@
 %%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 %%% DEALINGS IN THE SOFTWARE.
 %%%
-%%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2015-2017 Michael Truog
-%%% @version 1.7.3 {@date} {@time}
+%%% @author Michael Truog <mjtruog at protonmail dot com>
+%%% @copyright 2015-2019 Michael Truog
+%%% @version 1.8.0 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_service_monitoring).
--author('mjtruog [at] gmail (dot) com').
+-author('mjtruog at protonmail dot com').
 
 -behaviour(cloudi_service).
 
@@ -99,7 +99,6 @@
 
 %% cloudi_service callbacks
 -export([cloudi_service_init/4,
-         cloudi_service_handle_request/11,
          cloudi_service_handle_info/3,
          cloudi_service_terminate/3]).
 
@@ -271,9 +270,9 @@ add([_ | _] = ServiceConfig) ->
     end,
     case lists:keytake(options, 1, ServiceConfig) of
         {value, {_, Options}, NextServiceConfig} ->
-            NextServiceConfig ++ [{options, add_options(Type, Options)}];
+            [{options, add_options(Type, Options)} | NextServiceConfig];
         false ->
-            ServiceConfig ++ [{options, add_options(Type, [])}]
+            [{options, add_options(Type, [])} | ServiceConfig]
     end.
 
 %%-------------------------------------------------------------------------
@@ -644,12 +643,6 @@ cloudi_service_init(Args, _Prefix, _Timeout, Dispatcher) ->
                 nodes_hidden = erlang:length(erlang:nodes(hidden)),
                 nodes_all = erlang:length(erlang:nodes(connected))}}.
 
-cloudi_service_handle_request(_Type, _Name, _Pattern, _RequestInfo, _Request,
-                              _Timeout, _Priority, _TransId, _Pid,
-                              #state{} = State,
-                              _Dispatcher) ->
-    {reply, <<>>, State#state{}}.
-
 cloudi_service_handle_info(cloudi_update,
                            #state{service = Service,
                                   environment = EnvironmentLookup,
@@ -728,9 +721,10 @@ cloudi_service_handle_info({nodedown, _Node, OptionList},
     {_, NodeType} = lists:keyfind(node_type, 1, OptionList),
     {noreply, node_update(NodeType, -1, State)};
 cloudi_service_handle_info(Request, State, _Dispatcher) ->
-    ?LOG_WARN("Unknown info \"~p\"", [Request]),
-    {noreply, State}.
+    {stop, cloudi_string:format("Unknown info \"~w\"", [Request]), State}.
 
+cloudi_service_terminate(_Reason, _Timeout, undefined) ->
+    ok;
 cloudi_service_terminate(_Reason, _Timeout,
                          #state{interval = Interval}) ->
     ok = cloudi_service_monitoring_cloudi:services_terminate(Interval),
