@@ -8,7 +8,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2012-2017 Michael Truog <mjtruog at gmail dot com>
+%%% Copyright (c) 2012-2017 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -28,13 +28,13 @@
 %%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 %%% DEALINGS IN THE SOFTWARE.
 %%%
-%%% @author Michael Truog <mjtruog [at] gmail (dot) com>
+%%% @author Michael Truog <mjtruog at protonmail dot com>
 %%% @copyright 2012-2017 Michael Truog
-%%% @version 1.7.1 {@date} {@time}
+%%% @version 1.7.2 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(quickrand).
--author('mjtruog [at] gmail (dot) com').
+-author('mjtruog at protonmail dot com').
 
 %% external interface
 -export([seed/0,
@@ -43,7 +43,10 @@
          uniform_cache/2,
          strong_uniform/1,
          strong_uniform_range/2,
-         strong_float/0]).
+         strong_float/0,
+         strong_floatL/0,
+         strong_floatM/0,
+         strong_floatR/0]).
 
 -ifdef(ERLANG_OTP_VERSION_16).
 -else.
@@ -293,8 +296,7 @@ strong_uniform_range(Min, Max)
 
 %%-------------------------------------------------------------------------
 %% @doc
-%% ===Return an Erlang floating point random number (double-precision).===
-%% return a floating point value between 0.0 and 1.0, inclusive
+%% ===Return an Erlang double-precision random number with the range [0.0 .. 1.0].===
 %% @end
 %%-------------------------------------------------------------------------
 
@@ -304,9 +306,63 @@ strong_uniform_range(Min, Max)
 strong_float() ->
     % 53 bits maximum for double precision floating point representation
     % (need to use a maximum value of math:pow(2, 53) with extra bit,
-    %  i.e. 16#1fffffffffffff + 1)
-    <<I:53/unsigned-integer, Bit:1, _:2>> = crypto:strong_rand_bytes(7),
-    (I + Bit) * ?DBL_EPSILON_DIV2.
+    %  i.e. 1 + 16#1fffffffffffff)
+    <<Bit:1, I:53/unsigned-integer, _:2>> = crypto:strong_rand_bytes(7),
+    if
+        Bit == 1, I == 0 ->
+            1.0;
+        true ->
+            I * ?DBL_EPSILON_DIV2
+    end.
+
+%%-------------------------------------------------------------------------
+%% @doc
+%% ===Return an Erlang double-precision random number with the range [0.0 .. 1.0).===
+%% Left portion of the 0.0 to 1.0 range.
+%% @end
+%%-------------------------------------------------------------------------
+
+-spec strong_floatL() ->
+    float().
+
+strong_floatL() ->
+    <<I:53/unsigned-integer, _:3>> = crypto:strong_rand_bytes(7),
+    I * ?DBL_EPSILON_DIV2.
+
+%%-------------------------------------------------------------------------
+%% @doc
+%% ===Return an Erlang double-precision random number with the range (0.0 .. 1.0).===
+%% Middle portion of the 0.0 to 1.0 range.
+%% @end
+%%-------------------------------------------------------------------------
+
+-spec strong_floatM() ->
+    float().
+
+strong_floatM() ->
+    <<I:53/unsigned-integer, _:3>> = crypto:strong_rand_bytes(7),
+    if
+        I == 0 ->
+            % almost never executes this case, an additional function call
+            % is necessary to have a uniform distribution
+            strong_floatM();
+        true ->
+            I * ?DBL_EPSILON_DIV2
+    end.
+
+%%-------------------------------------------------------------------------
+%% @doc
+%% ===Return an Erlang double-precision random number with the range (0.0 .. 1.0].===
+%% Right portion of the 0.0 to 1.0 range.
+%% @end
+%%-------------------------------------------------------------------------
+
+-spec strong_floatR() ->
+    float().
+
+strong_floatR() ->
+    <<I:53/unsigned-integer, _:3>> = crypto:strong_rand_bytes(7),
+    (I + 1) * ?DBL_EPSILON_DIV2.
 
 %%%------------------------------------------------------------------------
 %%% Private functions
