@@ -19,7 +19,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2011-2017 Michael Truog <mjtruog at gmail dot com>
+%%% Copyright (c) 2011-2019 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -39,13 +39,13 @@
 %%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 %%% DEALINGS IN THE SOFTWARE.
 %%%
-%%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2011-2017 Michael Truog
-%%% @version 1.7.2 {@date} {@time}
+%%% @author Michael Truog <mjtruog at protonmail dot com>
+%%% @copyright 2011-2019 Michael Truog
+%%% @version 1.8.0 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_x_uuid).
--author('mjtruog [at] gmail (dot) com').
+-author('mjtruog at protonmail dot com').
 
 %% external interface
 -export([new/1,
@@ -89,6 +89,14 @@
 -ifdef(ERLANG_OTP_VERSION_19).
 -else.
 -define(ERLANG_OTP_VERSION_20_FEATURES, true).
+-ifdef(ERLANG_OTP_VERSION_20).
+-else.
+-ifdef(OTP_RELEASE). % Erlang/OTP >= 21.0
+% able to use -elif here
+-else.
+-error("Erlang/OTP version invalid").
+-endif.
+-endif.
 -endif.
 -endif.
 -endif.
@@ -117,6 +125,12 @@
 
 -include("cloudi_x_uuid.hrl").
 
+% Erlang Binary Term Format constants
+% info from http://erlang.org/doc/apps/erts/erl_ext_dist.html
+-define(TAG_VERSION, 131).
+-define(TAG_PID_EXT, 103).
+-define(TAG_NEW_PID_EXT, 88).
+
 %%%------------------------------------------------------------------------
 %%% External interface functions
 %%%------------------------------------------------------------------------
@@ -175,49 +189,42 @@ new(Pid, Options)
     % make the version 1 UUID specific to the Erlang node and pid
 
     % 48 bits for the first MAC address found is included with the
-    % distributed Erlang node name
-    <<NodeD01, NodeD02, NodeD03, NodeD04, NodeD05,
-      NodeD06, NodeD07, NodeD08, NodeD09, NodeD10,
-      NodeD11, NodeD12, NodeD13, NodeD14, NodeD15,
-      NodeD16, NodeD17, NodeD18, NodeD19, NodeD20>> =
-      crypto:hash(sha, erlang:list_to_binary(MacAddress ++
-                                             erlang:atom_to_list(node()))),
-    % later, when the pid format changes, handle the different format
-    ExternalTermFormatVersion = 131,
-    PidExtType = 103,
-    <<ExternalTermFormatVersion:8,
-      PidExtType:8,
-      PidBin/binary>> = erlang:term_to_binary(Pid),
-    % 72 bits for the Erlang pid
-    <<PidID1:8, PidID2:8, PidID3:8, PidID4:8, % ID (Node specific, 15 bits)
-      PidSR1:8, PidSR2:8, PidSR3:8, PidSR4:8, % Serial (extra uniqueness)
-      PidCR1:8                       % Node Creation Count
-      >> = binary:part(PidBin, erlang:byte_size(PidBin), -9),
-    % reduce the 160 bit NodeData checksum to 16 bits
-    NodeByte1 = ((((((((NodeD01 bxor NodeD02)
-                       bxor NodeD03)
-                      bxor NodeD04)
-                     bxor NodeD05)
-                    bxor NodeD06)
-                   bxor NodeD07)
-                  bxor NodeD08)
-                 bxor NodeD09)
-                bxor NodeD10,
-    NodeByte2 = (((((((((NodeD11 bxor NodeD12)
-                        bxor NodeD13)
-                       bxor NodeD14)
-                      bxor NodeD15)
-                     bxor NodeD16)
-                    bxor NodeD17)
-                   bxor NodeD18)
-                  bxor NodeD19)
-                 bxor NodeD20)
-                bxor PidCR1,
-    % reduce the Erlang pid to 32 bits
-    PidByte1 = PidID1 bxor PidSR4,
-    PidByte2 = PidID2 bxor PidSR3,
-    PidByte3 = PidID3 bxor PidSR2,
-    PidByte4 = PidID4 bxor PidSR1,
+    % distributed Erlang node name to be hashed with the node creation count
+    NodeData = [MacAddress, erlang:atom_to_binary(node(), utf8)],
+
+    % Reduce the node information to 16 bits and the pid information to 32 bits
+    % for use as the UUID v1 Node Id
+    {NodeCreationCount, PidData} = case erlang:term_to_binary(Pid) of
+        % (when the pid format changes, handle the different formats)
+        <<?TAG_VERSION:8,
+          ?TAG_PID_EXT:8,PidBin/binary>> ->
+            % 72 bits for the Erlang pid
+            <<PidID1:8,PidID2:8,PidID3:8,PidID4:8,% ID (Node specific, 15 bits)
+              PidSR1:8,PidSR2:8,PidSR3:8,PidSR4:8,% Serial (extra uniqueness)
+              PidCR:8                             % Node Creation Count
+              >> = binary:part(PidBin, erlang:byte_size(PidBin), -9),
+            {PidCR,
+             [PidID1, PidID2, PidID3, PidID4,
+              PidSR1, PidSR2, PidSR3, PidSR4]};
+        % format supported in Erlang/OTP 19.0-rc1
+        % required for Erlang/OTP 23.0 (and Erlang/OTP 22.0-rc2)
+        <<?TAG_VERSION:8,
+          ?TAG_NEW_PID_EXT:8,PidBin/binary>> ->
+            % 96 bits for the Erlang pid
+            <<PidID1:8,PidID2:8,PidID3:8,PidID4:8,% ID (Node specific, 15 bits)
+              PidSR1:8,PidSR2:8,PidSR3:8,PidSR4:8,% Serial (extra uniqueness)
+              PidCR:32                            % Node Creation Count
+              >> = binary:part(PidBin, erlang:byte_size(PidBin), -12),
+            {PidCR,
+             [PidID1, PidID2, PidID3, PidID4,
+              PidSR1, PidSR2, PidSR3, PidSR4]}
+    end,
+    Node32 = cloudi_x_quickrand_hash:jenkins_32(NodeData, NodeCreationCount),
+    Node16 = (Node32 bsr 16) bxor (Node32 band 16#FFFF),
+    Pid32 = cloudi_x_quickrand_hash:jenkins_32(PidData),
+    NodeId = <<Node16:16/big-unsigned-integer,
+               Pid32:32/big-unsigned-integer>>,
+
     ClockSeq = pseudo_random(16384) - 1,
     TimestampTypeInternal = if
         TimestampType =:= os ->
@@ -235,9 +242,7 @@ new(Pid, Options)
             end
     end,
     TimestampLast = timestamp(TimestampTypeInternal),
-    #uuid_state{node_id = <<NodeByte1:8, NodeByte2:8,
-                            PidByte1:8, PidByte2:8,
-                            PidByte3:8, PidByte4:8>>,
+    #uuid_state{node_id = NodeId,
                 clock_seq = ClockSeq,
                 timestamp_type = TimestampTypeInternal,
                 timestamp_last = TimestampLast}.
@@ -913,6 +918,8 @@ string_to_uuid(N01, N02, N03, N04, N05, N06, N07, N08,
 -spec is_uuid(any()) ->
     boolean().
 
+is_uuid(<<0:128>>) ->
+    true;
 is_uuid(<<_:48,
           Version:4/unsigned-integer,
           _:12,
@@ -1016,6 +1023,7 @@ mac_address() ->
     ok.
 
 test() ->
+    true = cloudi_x_uuid:is_uuid(<<0:128>>),
     % version 1 tests
     % uuidgen -t ; date
     % "Fri Dec  7 19:13:58 PST 2012"
@@ -1365,7 +1373,7 @@ mac_address([{_, L} | Rest]) ->
 
 -ifdef(ERLANG_OTP_VERSION_18_FEATURES).
 pseudo_random(N) ->
-    % assuming exsplus for 58 bits, period 8.31e34
+    % assuming exsp/exsplus for 58 bits, period 8.31e34
     rand:uniform(N).
 -else.
 pseudo_random(N) ->
