@@ -4,7 +4,7 @@
 #
 # MIT License
 #
-# Copyright (c) 2017 Michael Truog <mjtruog at gmail dot com>
+# Copyright (c) 2017-2019 Michael Truog <mjtruog at protonmail dot com>
 #
 # Permission is hereby granted, free of charge, to any person obtaining a
 # copy of this software and associated documentation files (the "Software"),
@@ -24,11 +24,20 @@
 # FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 # DEALINGS IN THE SOFTWARE.
 #
+"""
+Count Integration Test with Python
+"""
 
-import sys, threading, types, traceback
-from cloudi import API, terminate_exception
+from __future__ import print_function
+import sys
+import threading
+import traceback
+from cloudi import API, TerminateException
 
 class Task(threading.Thread):
+    """
+    count thread task
+    """
     def __init__(self, api, name, exception):
         threading.Thread.__init__(self)
         self.__api = api
@@ -37,19 +46,25 @@ class Task(threading.Thread):
         self.__count = 0
 
     def run(self):
+        """
+        run the count thread
+        """
+        # pylint: disable=bare-except
         try:
-            self.__api.subscribe(self.__name + '/get', self.request)
+            self.__api.subscribe(self.__name + '/get', self.__request)
 
             result = self.__api.poll()
-            assert result == False
+            assert result is False
         except self.__terminate_exception:
             pass
         except:
             traceback.print_exc(file=sys.stderr)
         print('terminate count %s' % self.__name)
 
-    def request(self, request_type, name, pattern, request_info, request,
-                timeout, priority, trans_id, pid):
+    def __request(self, request_type, name, pattern, request_info, request,
+                  timeout, priority, trans_id, pid):
+        # pylint: disable=unused-argument
+        # pylint: disable=too-many-arguments
         if self.__count == 4294967295:
             self.__count = 0
         else:
@@ -60,14 +75,16 @@ class Task(threading.Thread):
                            b'', response,
                            timeout, trans_id, pid)
 
-if __name__ == '__main__':
+def _main():
     thread_count = API.thread_count()
     assert thread_count >= 1
-    
-    threads = [Task(API(i), 'python', terminate_exception)
-               for i in range(thread_count)]
-    for t in threads:
-        t.start()
-    for t in threads:
-        t.join()
 
+    threads = [Task(API(thread_index), 'python', TerminateException)
+               for thread_index in range(thread_count)]
+    for thread in threads:
+        thread.start()
+    for thread in threads:
+        thread.join()
+
+if __name__ == '__main__':
+    _main()
