@@ -8,7 +8,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2011-2018 Michael Truog <mjtruog at protonmail dot com>
+%%% Copyright (c) 2011-2019 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -29,8 +29,8 @@
 %%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog at protonmail dot com>
-%%% @copyright 2011-2018 Michael Truog
-%%% @version 1.7.4 {@date} {@time}
+%%% @copyright 2011-2019 Michael Truog
+%%% @version 1.8.0 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_service_test_msg_size).
@@ -139,7 +139,8 @@ cloudi_service_init(_Args, Prefix, _Timeout, Dispatcher) ->
     cloudi_service:subscribe(Dispatcher, "erlang"),
     {ok, StateN}.
 
-cloudi_service_handle_request(_Type, _Name, Pattern, RequestInfo, Request,
+cloudi_service_handle_request(_RequestType, _Name, Pattern,
+                              RequestInfo, Request,
                               Timeout, Priority, _TransId, _Pid,
                               #state{suffixes = [Suffix | Suffixes]} = State,
                               _Dispatcher) ->
